package eu.jonahbauer.raytracing.render.image;

import eu.jonahbauer.raytracing.render.canvas.Canvas;
import eu.jonahbauer.raytracing.render.color.ColorSpace;
import eu.jonahbauer.raytracing.render.color.ColorSpaces;
import org.jetbrains.annotations.NotNull;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

public class PPMImageWriter implements ImageWriter {
    public static final PPMImageWriter sRGB = new PPMImageWriter(ColorSpaces.sRGB);

    private final @NotNull ColorSpace cs;

    public PPMImageWriter(@NotNull ColorSpace cs) {
        this.cs = Objects.requireNonNull(cs, "cs");
    }

    @Override
    public void write(@NotNull Canvas image, @NotNull OutputStream out) throws IOException {
        try (var writer = new BufferedWriter(new OutputStreamWriter(out, StandardCharsets.US_ASCII))) {
            writer.write("P3\n");
            writer.write(String.valueOf(image.getWidth()));
            writer.write(" ");
            writer.write(String.valueOf(image.getHeight()));
            writer.write("\n255\n");

            for (int y = 0; y < image.getHeight(); y++) {
                for (int x = 0; x < image.getWidth(); x++) {
                    var color = cs.encode(image.getRGB(x, y, cs));
                    writer.write(String.valueOf(color.red()));
                    writer.write(" ");
                    writer.write(String.valueOf(color.green()));
                    writer.write(" ");
                    writer.write(String.valueOf(color.blue()));
                    writer.write("\n");
                }
            }
        }
    }
}
